<?php

function forward_post_handler() {

    // Get the new forward address, if any
    $newAddress = getvar('newAddress');

    // See what address should be removed, if any
    $removeAddress = getvar('removeAddress');
    
    // Is the user disabling forwarding?
    $disable = getvar('disable');
    
    // Is the user keeping a local copy?
    $localCopy = getvar('localCopy');
    
    $success = true;
    $msg = NULL;

    if (!$disable) {
        // Did we get an address?
        if (!$newAddress && !$removeAddress && !$localCopy) {
            $msg = handle_error(7);
            $success = false;
        }
        elseif (!$removeAddress) {
            // Is the forwarding address valid?
            $emailValid = email_valid($newAddress);
            if (!$emailValid) {
                $msg = handle_error(5);
                $success = false;
            }
            $mailhost = explode('@', $newAddress);
            $mailhost = $mailhost[1];
            $hostValid = host_valid($mailhost);
            if (!$hostValid) {
                $msg = handle_error(5);
                $success = false;
            }
        }
    }

    if ($success) {
        commit_forward($newAddress, $disable, $removeAddress);
    }
    else {
        $response = array();
        $response['message'] = '<div id="webtools-response" class="ui-state-error ui-corner-all">'.$msg.'</div>';
        $response['addr_list'] = '';
        $response['hidden_input'] = '';
        $response['update_content'] = 'no';
        $response['forward_disabled'] = 'no';
        send_response($response);
    }
}   

function commit_forward($newAddress, $disable, $removeAddress) {

    $rcmail = rcmail::get_instance();

    $USERNAME = $rcmail->config->get('USERNAME');
    $HOME = $rcmail->config->get('HOME');
    $RUNAS_CMD = $rcmail->config->get('RUNAS_CMD');
    $MF = $rcmail->config->get('MF');

    // Get some needed POST variables
    $forwardType = getvar('forwardType');
    $vacation = getvar('vacation');
    $newFormatVacation_S = getvar('newFormatVacation_S');    

    // Initialize other variables
    $insert = '';
    $createForward = true;
    $existingAddr_Arr = array();
    $type = '';
    $oldemail = '';
    $msg = '';
    $status = 0;
    $dupe = 0;
    $user_cmd = '';
    $success = true;
    $hasVacation = false;
    $newVacationFileExist = false;

    // This is the response that will be sent back to the client
    $response = array('message' => '', 'hidden_input' => '', 'addr_list' => '', 'update_content' => 'yes', 'forward-disabled' => 'no');

    // Set insert to display some rutgers specific comments
    $insert = "################################################\n";
    $insert .= "# THIS FILE WAS CREATED BY A WEBTOOL.          #\n";
    $insert .= "# DO NOT EDIT THIS FILE BY HAND.               #\n";
    $insert .= "################################################\n\n";

    // Does the user have an existng vacation rule in an old-format file?
    if ($forwardType && $vacation && !$newFormatVacation_S) {
        $insert .= getVacationRule();
        $hasVacation = true;
    }

    // Are we adding on top of new formatted forward file?
    if ($newFormatVacation_S) {
        $hold = unserialize(base64_decode($newFormatVacation_S));
        $insert .= getVacationRule(NULL, $hold);
        $hasVacation = true;
    }

    if ($disable) {
        if (!$hasVacation) {
            $createForward = false;
        }
    }
    else {
        // Is the user keeping a local copy?
        $localCopy = NULL;
        $localCopyPrior = getvar('localCopyPrior');
        if ($removeAddress) {
            // Don't change the local copy setting if just removing an address
            $localCopy = $localCopyPrior;
        }
        else {
            $localCopy = getvar('localCopy');
        }

        // Get existing forward addresses
        $existingAddr_S = getvar('existingAddr_S');
        if (isset($existingAddr_S)) {
            $existingAddr_Arr = unserialize(base64_decode($existingAddr_S));
        }
        
        // See if the user has a (deprecated) mailfilter-vacation file
        $newVacationFileExist = getvar('newVacationFileExist');
        
        if (!$removeAddress) {

            // Set server information
            $SERVER_NAME = $_SERVER['SERVER_NAME'];
            $IP = gethostbyname($SERVER_NAME);
            $server = gethostbyaddr($IP);

            // Prevent people from forwarding to themselves
            $oldemail = ("$USERNAME@$server");
            for ($index = 0; $index < count($existingAddr_Arr); $index++) {
                if (strtolower($existingAddr_Arr[$index]) == strtolower($oldemail)) {
                    $success = false;
                    $createForward = false;
                    $dupe = 1;
                    $msg .= 'The forwarding address is the same as your current email address!<br />';
                }
            }
            
            // Prevent people from forwarding to duplicate emails
            for ($index = 0; $index < count($existingAddr_Arr); $index++) {
                if (strtolower($existingAddr_Arr[$index]) == strtolower($newAddress)) {
                    $success = false;
                    $createForward = false;
                    $dupe = 1;
                    $msg .= 'You are already forwarding your email to that specified address!<br />';
                }
            }

            // Finally add the new address to the all addresses array
            array_push($existingAddr_Arr, $newAddress);
        }
        else {
            
            $found = 0;

            // Remove the desired address
            for ($index = 0; $index < count($existingAddr_Arr); $index++) {
                if (strtolower($existingAddr_Arr[$index]) == strtolower($removeAddress)) {
                    array_splice($existingAddr_Arr, $index, 1);
                    $found = 1;
                    break;
                }
            }
                      
            if (!$found) {
                $success = false;
                $createForward = false;
                $msg .= 'No addresses have been removed!<br />';
            }
            else {
                if (empty($existingAddr_Arr)) {
                    $disable = true; 
                    if (!$hasVacation) {
                        $createForward = false;
                    }
                }
                $msg .= 'The address has been successfully removed!<br /><br />';
            }

        }

        if (!$disable) {
            // Are we leaving a local copy?
            if ((!$removeAddress && $localCopy === 'on') || ($removeAddress && $localCopyPrior === 'on')) {
                $addresses = implode(' ', $existingAddr_Arr);
                $insert .= "cc \"!$addresses\"\n";
            }
            else {
                $addresses = implode(' ', $existingAddr_Arr);
                $insert .= "to \"!$addresses\"\n";
            }
        }

    }

    // Should we write the new forward file?
    if ($createForward) {
        $user_cmd = "$MF Maildir/mailfilter-forward";
        $exec_target = "$RUNAS_CMD $USERNAME $user_cmd";
        unipipe_exec($exec_target, $insert, $status);
        if ($status != 0) {
            $msg .= 'Could not create your new forward/vacation file.<br />';
            $success = false;
        }
        elseif (!$disable) {
            $msg .= 'Your new forward file was created.<br />';
        }
        else {
            $msg .= 'Mail forwarding has been successfully disabled.<br />';
            $response['disabled'] = 'yes';
        }
    }
    elseif ($success && $disable) {
        // No vacation rule, so just remove the file
        $user_cmd = "find $HOME -maxdepth 2 -path $HOME/Maildir/mailfilter-forward -exec rm -f '{}' \;";
        $exec_target = "$RUNAS_CMD $USERNAME $user_cmd";
        exec($exec_target, $result, $status);
        if ($status != 0) {
            $msg .= 'Tried removing your forward file, but failed.<br />';
            $success = FALSE;
        }
        else {
            $msg .= 'Forwarding has been disabled and your forward file has been removed.<br />';
            $response['disabled'] = 'yes';
        }
    }

    // Move the old-format vacation file if it exists
    if ($success && $newVacationFileExist) {
        $user_cmd = "find $HOME -maxdepth 2 -path $HOME/Maildir/mailfilter-vacation -exec mv '{}' $HOME/Maildir/mailfilter-vacation-old \;";
        $exec_target = "$RUNAS_CMD $USERNAME $user_cmd";
        exec($exec_target, $result, $status);
        if ($status != 0) {
            $msg .= 'Could not move your old mailfilter-vacation file to mailfilter-vacation-old<br />';
            $success = false;
        }
        else {
            $msg .= 'Moved your old mailfilter-vacation file to mailfilter-vacation-old<br />';
        }
    }
        
    // Move the old-format forward file if it exists
    if ($success && $forwardType) {
        switch($forwardType) {
            case 'f':
                $user_cmd = "find $HOME -maxdepth 1 -name .forward -exec mv '{}' $HOME/.forward-old-webtool \;";
                $type = '.forward';
                break;
            case 'q':
                    $user_cmd = "find $HOME -maxdepth 1 -name .qmail -exec mv '{}' $HOME/.qmail-old-webtool \;";
                    $type = '.qmail';
                    break;
        }

        $exec_target = "$RUNAS_CMD $USERNAME $user_cmd";
        exec($exec_target, $result, $status);

        if ($status != 0) {
            $msg .= "Could not move your old $type file to $type-old-webtool.<br />";
            $success = false;
        }
        else {
            $msg .= "Your old $type file has been moved to $type-old-webtool.<br />";
        }
    }

    if ($success) {
        $response['message'] = '<div id="webtools-response" class="ui-state-highlight ui-corner-all">Thank you '.$USERNAME.',<br /><br />'.$msg;
        
        if (!$disable) {
            // Update the address list
            $response['addr_list'] = '<label style="display: block; margin-bottom: 10px;">Existing Addresses</label><ul>';
            
            foreach ($existingAddr_Arr as $address) {
                $response['addr_list'] .= 
                    '<li onclick="forward_post(false, \''.$address.'\'); return false;" title="Remove this address">'
                        .$address.
                    '</li>';

            }
            $response['addr_list'] .= '<li onclick="forward_post(true); return false;" title="Remove all addresses">Remove All</li></ul>';

            $existingAddr_S = base64_encode(serialize($existingAddr_Arr));
            $response['hidden_input'] .= '<input type="hidden" name="existingAddr_S" value="'.$existingAddr_S.'" />';
            $response['hidden_input'] .= '<input type="hidden" id="localCopyPrior" name="localCopyPrior" value="'.$localCopy.'" />';
        }
    }
    else {
        $response['message'] = '<div id="webtools-response" class="ui-state-error ui-corner-all">Sorry '.$USERNAME.',<br /><br />'.$msg;
        $response['update_content'] = 'no';
    }
    $response['message'] .= '</div>';

    send_response($response);
}

function send_response($response) {
    $rcmail = rcmail::get_instance();
    $rcmail->output->command('plugin.forward-post-callback', $response);    
}

?>
